import Ember from 'ember';

const {Component, computed, observer} = Ember;


export default Component.extend({
  classNames: ['modal', 'fade'],
  afterCloseClass: 'hidden.bs.modal',
  closeClass: 'hide.bs.modal',
  keyboard: true,
  backdrop: true,
  closeAfterSaveSuccess: true,
  closeAfterSaveFail: false,
  disabledSubmit: false,
  disabledClose: false,
  staticBackdrop: false,
  afterClose: 'afterClose',
  submit: 'submit',
  beforeClose: 'beforeClose',
  isReadyToClose: false,

  backdropSetting: computed('backdrop', 'staticBackdrop', function() {
    var res = this.get('backdrop');

    return (res && this.get('staticBackdrop')) ? 'static' : res;
  }),

  modalOptions: computed('backdropSetting', 'keyboard', function() {

    return {
        backdrop: this.get('backdropSetting'),
        keyboard: this.get('keyboard')
    };
  }),

  actions: {

    submit() {
      var self = this,
        defer = Ember.RSVP.defer();

        self.set('disabledSubmit', true);

        defer.promise.then(
          function() {
            self.set('disabledSubmit', false);
        }).then(
          function() {
            if(self.get('closeAfterSaveSuccess')) {
                self.set('isReadyToClose', true);
                self.$().modal('hide');
            }
        }, function() {
            if(self.get('closeAfterSaveFail')) {
                self.set('isReadyToClose', true);
                self.$().modal('hide');
            }
        });

        self.sendAction(self.get('submit'), defer);
    }

  },

  showChanged: observer('isVisible', function() {
    if(this.get('isVisible')) {
      this.renderModal();

    } else {
      this.destroyModal();
    }
  }),

  renderModal: function() {
    var self = this;
    
    self.$().modal(self.get('modalOptions'));

    self.$().modal().on(self.get('closeClass'), function(e) {
        Ember.run(function() {
          e.stopImmediatePropagation();

          self.set('disabledClose', true);
          
          if(self.get('isReadyToClose')) { return true; }
          
          e.preventDefault();

          var defer = Ember.RSVP.defer();

          defer.promise.then(
            function() {
              self.set('isReadyToClose', true);
              self.$().modal('hide');
            }, 
            function() {
              self.setProperties({
                disabledClose: false,
                isReadyToClose: false
              });
            }
          );
          self.sendAction(self.get('beforeClose'), defer);
        });

    });

    self.$().modal().on(self.get('afterCloseClass'), 
      function() {
        Ember.run(function() {
          self.destroyModal();
          self.setProperties({
              isVisible: false,
              isReadyToClose: false,
              disabledClose: false
          });
          self.sendAction(self.get('afterClose'));
        });
      }
    );

  },

  destroyModal: function() {

    this.$().off(this.get('afterCloseClass'));
  },
  
  setupBootstrapModal: function() {
    
    this._super();
    Ember.run.scheduleOnce('afterRender', this, this.afterRenderEvent);
  }.on('didInsertElement'),

  afterRenderEvent: function() {
    
    if(this.get('isVisible')) {
        this.renderModal();
    }
  },

  teardownBootstrapModal: function() {

    this.destroyModal();

  }.on('willDestroyElement')

});
